@@ -1,9 +1,9 @@
 """NAME.py : Description of the file
 
-* Copyright: 2022 Sampsa Riikonen
+* Copyright: 2021-2022 Sampsa Riikonen
 * Authors  : Sampsa Riikonen
-* Date     : 1/2022
-* Version  : 0.0.2
+* Date     : 6/2021
+* Version  : 0.0.3
 
 This file is part of the task_thread library
 
@@ -15,6 +15,7 @@ import sys
 import logging
 import asyncio
 
+
 def test1():
     st = """Empty test
     """
@@ -32,12 +33,12 @@ def test2():
 def main():
     pre = __name__ + "main :"
     print(pre, "main: arguments: ", sys.argv)
-    if (len(sys.argv) < 2):
+    if len(sys.argv) < 2:
         print(pre, "main: needs test number")
     else:
         st = "test" + str(sys.argv[1]) + "()"
         exec(st)
 
 
-if (__name__ == "__main__"):
+if __name__ == "__main__":
     main()
